using System.Collections;
using System.Collections.Generic;
using Cuyahoga.Core.Domain;

namespace Cuyahoga.Core.Service.Membership
{
	/// <summary>
	/// Provides functionality for user management.
	/// </summary>
	public interface IUserService
	{
		/// <summary>
		/// Find users by name (username or last name, starting with). When no searchString is provided, all users are returned.
		/// </summary>
		/// <param name="searchString"></param>
		/// <returns></returns>
		IList<User> FindUsersByName(string searchString);

		/// <summary>
		/// Get a user by ID.
		/// </summary>
		/// <param name="userId"></param>
		/// <returns></returns>
		User GetUserById(int userId);

		/// <summary>
		/// Get a user by username and email.
		/// </summary>
		/// <param name="username"></param>
		/// <param name="email"></param>
		/// <returns></returns>
		User GetUserByUsernameAndEmail(string username, string email);

		/// <summary>
		/// Create a new user with the supplied username and email address for the given site.
		/// </summary>
		/// <param name="username"></param>
		/// <param name="email"></param>
		/// <param name="site"></param>
		/// <returns>The generated password in clear text.</returns>
		string CreateUser(string username, string email, Site site);

		/// <summary>
		/// Update an existing user.
		/// </summary>
		/// <param name="user"></param>
		void UpdateUser(User user);

		/// <summary>
		/// Delete an existing user.
		/// </summary>
		/// <param name="user"></param>
		void DeleteUser(User user);

		/// <summary>
		/// Reset the password of the user with the given username and email address.
		/// </summary>
		/// <param name="username"></param>
		/// <param name="email"></param>
		/// <returns>The new password</returns>
		string ResetPassword(string username, string email);

		/// <summary>
		/// Get all available roles.
		/// </summary>
		/// <returns></returns>
		IList<Role> GetAllRoles();

		/// <summary>
		/// Get all roles for the given role id's.
		/// </summary>
		/// <param name="roleIds"></param>
		/// <returns></returns>
		IList<Role> GetRolesByIds(int[] roleIds);

		/// <summary>
		/// Get a single role by id.
		/// </summary>
		/// <param name="roleId"></param>
		/// <returns></returns>
		Role GetRoleById(int roleId);

		/// <summary>
		/// Get all available rights.
		/// </summary>
		/// <returns></returns>
		IList<Right> GetAllRights();

		/// <summary>
		/// Get a Right by id.
		/// </summary>
		/// <param name="rightId"></param>
		/// <returns></returns>
		Right GetRightById(int rightId);
	}
}
