﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Compilation;
using System.Reflection;
using System.Web.Mvc;

namespace Cuyahoga.Web.MVC.ViewComponents
{
	/// <summary>
	/// Base class for the view component factories. Currently it provides the functionality for mapping component names
	/// to component types.
	/// </summary>
	public abstract class AbstractViewComponentFactory : IViewComponentFactory
	{
		private static object _locker = new object();
		private Dictionary<string, Type> _nameTypesCache = null;

		/// <summary>
		/// Resolve the type of a component with a given name. There is a limitation that the names of the 
		/// view components have to be unique in the application.
		/// </summary>
		/// <param name="componentName"></param>
		/// <returns></returns>
		protected virtual Type ResolveType(string componentName)
		{
			if (this._nameTypesCache == null)
			{
				InitNameTypesCache();
			}
			if (!this._nameTypesCache.ContainsKey(componentName))
			{
				throw new ArgumentException(String.Format("No view component found with name '{0}'", componentName));
			}
			return this._nameTypesCache[componentName];
		}

		private void InitNameTypesCache()
		{
			lock (_locker)
			{
				this._nameTypesCache = new Dictionary<string, Type>();
	
				foreach (Assembly assembly in BuildManager.GetReferencedAssemblies())
				{
					foreach (Type type in assembly.GetTypes())
					{
						if (IsViewComponent(type))
						{
							// Store name without the 'Component' part. 
							string name = type.Name.Substring(0, type.Name.Length - 9);
							this._nameTypesCache[name] = type;
						}
					}
				}
			}
		}

		protected virtual bool IsViewComponent(Type type)
		{
			return (((type.Name.EndsWith("Component", StringComparison.OrdinalIgnoreCase) 
				&& (type != typeof(IViewComponent))) 
				&& (type != typeof(ViewComponent))) 
				&& typeof(IViewComponent).IsAssignableFrom(type));
		}

		#region IViewComponentFactory Members

		public abstract IViewComponent Create(string componentName);

		#endregion
	}
}
