﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.IO;
using System.Web;

namespace Cuyahoga.Web.MVC.ViewComponents
{
	/// <summary>
	/// A reusable self-containing component for the ASP.NET MVC framework. 
	/// </summary>
	/// <remarks>
	/// The main reason for these components is a better separation of logic and view for partial views 
	/// and the possibility to hook these guys up with an IoC container for dependency injection. 
	/// View components are best suited for partial views that contain quite a bit logic, but have to act
	/// independent from the current controller context.
	/// </remarks>
	public abstract class ViewComponent : IViewComponent
	{
		private const string contextDataKey = "ContextData";
		private ViewContext _viewContext;
		private Controller _controller;
		private readonly IDictionary<string, object> _viewComponentData = new Dictionary<string, object>();

		/// <summary>
		/// The ASP.NET MVC ViewContext for the current request.
		/// </summary>
		protected virtual ViewContext CurrentViewContext
		{
			get
			{
				if (this._viewContext == null)
				{
					throw new InvalidOperationException("ViewContext is not available at this time. Make sure the view component is initialized before usage.");
				}
				return this._viewContext;
			}
		}

		/// <summary>
		/// The ASP.NET MVC Controller for the current request.
		/// </summary>
		/// <remarks>
		/// We need the Controller instance instead of IController because otherwise it seems impossible get the instance of
		/// the current view factory.
		/// </remarks>
		protected virtual Controller CurrentController
		{
			get
			{
				if (this._controller == null)
				{
					throw new InvalidOperationException("Controller is not available at this time. Make sure the view component is initialized before usage.");
				}
				return this._controller;
			}
		}

		/// <summary>
		/// HACK: we need to store view component data outside the 'normal' MVC viewdata bag because it is not available to
		/// webform ViewUserControls that are created without a parent ViewPage. View User Controls should depend on ViewContext
		/// for this, instead of ViewPage, but it's a limitation of the CTP version.
		/// </summary>
		protected virtual IDictionary<string, object> ViewComponentData
		{
			get	{ return this._viewComponentData; }
		}

		/// <summary>
		/// The name of the view component.
		/// </summary>
		public string Name
		{
			get { return GetComponentName(); }
		}

		#region IViewComponent Members

		/// <summary>
		/// Initialize the view component. This method is called by the framework.
		/// </summary>
		/// <param name="viewContext"></param>
		/// <param name="contextData"></param>
		public void Init(ViewContext viewContext, object contextData)
		{
			this._viewContext = viewContext;
			// Cast to Controller because we can't do much with the IController interface and ViewContext doesn't provide enough 
			// information such as ViewData. 
			this._controller = viewContext.Controller as Controller;

			// Store the context view data
			this._viewComponentData[contextDataKey] = contextData;
			
			Initialize();
		}

		/// <summary>
		/// Initialize the view component. Override this method to add your own initialization logic.
		/// </summary>
		public virtual void Initialize()
		{ 
		}

		/// <summary>
		/// Render the view of the component to a string.
		/// </summary>
		public virtual string Render()
		{
			// It would be really nice to have a method on IView that returns the rendered output as a string.
			// For the time being, we have to resort to a hack where the TextWriter of the output response
			// is replaced temporarily with a local StringWriter to intercept the output of the view.

			IHttpContext context = this.CurrentViewContext.HttpContext;
			TextWriter oldWriter = context.Response.Output;
			StringBuilder sb = new StringBuilder();
			using (TextWriter componentStringWriter = new StringWriter(sb))
			{
				context.Response.SwitchWriter(componentStringWriter);
				// Is there another way to get hold of the current ViewFactory? 
				IViewFactory viewFactory = CurrentController.ViewFactory;

				// Create the view component with the view component data. 
				// If viewcomponent data only contains one entry, the value of this entry is passed into the view factory, otherwise
				// the IDictionary is passed to the view factory. This way it's still possible to use strong-typed views.
				object viewData = null;
				if (this._viewComponentData.Count == 1)
				{
					foreach (object viewDataObject in this._viewComponentData.Values)
					{
						viewData = viewDataObject;
					}
				}
				else
				{
					viewData = this._viewComponentData;
				}
				// Create the view. Note that we're only dealing with the interface here, so every view engine can be leveraged.
				IView viewComponentView = viewFactory.CreateView(CurrentController.ControllerContext, GetViewName(), null, viewData);
				viewComponentView.RenderView(this._viewContext);

				context.Response.SwitchWriter(oldWriter);
			}
			return sb.ToString();
		}

		#endregion

		private string GetViewName()
		{
			// ViewComponent views are located in ~/Views/Shared/Components/ and and have the same name as the 
			// component without the 'component' suffix.
			// The view engine always looks in ~/Views/Shared/ so we're adding 'Components/ComponentName to that.
			return "Components/" + this.Name; 
		}

		private string GetComponentName()
		{
			// Get the component name. It is extracted from the type and should end with 'component'.
			string typeName = this.GetType().Name;
			if (! typeName.EndsWith("component", StringComparison.OrdinalIgnoreCase))
			{
				throw new Exception("The view component should have a name that ends with 'Component' by convention.");
			}
			string typeNameWithoutComponent = typeName.Substring(0, typeName.Length - 9);

			return typeNameWithoutComponent;
		}
	}
}
