﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Web.Mvc;

namespace Cuyahoga.Web.MVC.ViewComponents
{
	/// <summary>
	/// Singleton for retrieving view components. Based on ASP.NET MVC's ControllerBuilder.
	/// </summary>
	public class ViewComponentBuilder
	{
		private static ViewComponentBuilder _instance = new ViewComponentBuilder();
		private Dictionary<string, IViewComponentFactory> _viewComponentFactoryRegistry = new Dictionary<string,IViewComponentFactory>();
		private IViewComponentFactory _defaultViewComponentFactory; 
		private static object _locker = new object();

		/// <summary>
		/// Get the instance of ViewComponentBuilder.
		/// </summary>
		public static ViewComponentBuilder Current
		{
			get { return _instance; }
		}

		/// <summary>
		/// The default view component factory. All view components that are not registered via
		/// SetViewComponentFactory() will be created by this view component factory.
		/// </summary>
		internal IViewComponentFactory DefaultViewComponentFactory
		{
			get
			{
				if (this._defaultViewComponentFactory == null)
				{
					this._defaultViewComponentFactory = new DefaultViewComponentFactory();
				}
				return this._defaultViewComponentFactory;
			}
		}

		/// <summary>
		/// Create an instance of the view component with the given name.
		/// </summary>
		/// <param name="componentName"></param>
		/// <returns></returns>
		public IViewComponent CreateViewComponent(string componentName)
		{
			// View component names are case-insensitive.
			string lowerCaseComponentName = componentName.ToLower(CultureInfo.InvariantCulture);

			// Try to find a factory for the component. If not found, use the default factory.
			if (this._viewComponentFactoryRegistry.ContainsKey(lowerCaseComponentName))
			{
				return this._viewComponentFactoryRegistry[lowerCaseComponentName].Create(componentName);
			}
			else
			{
				return this.DefaultViewComponentFactory.Create(componentName);
			}
		}

		/// <summary>
		/// Set the default view component factory. 
		/// </summary>
		/// <param name="defaultViewComponentFactory"></param>
		public void SetDefaultViewComponentFactory(IViewComponentFactory defaultViewComponentFactory)
		{
			this._defaultViewComponentFactory = defaultViewComponentFactory;
		}

		/// <summary>
		/// Register a view component that has to be created with a specific view component factory.
		/// </summary>
		/// <param name="componentName"></param>
		/// <param name="viewComponentFactory"></param>
		public void SetViewComponentFactory(string componentName, IViewComponentFactory viewComponentFactory)
		{
			if (this._viewComponentFactoryRegistry.ContainsKey(componentName.ToLower(CultureInfo.InvariantCulture)))
			{
				throw new ArgumentException(String.Format("A view component with the name '{0}' is already registered.", componentName));
			}
			lock (_locker)
			{
				this._viewComponentFactoryRegistry.Add(componentName.ToLower(CultureInfo.InvariantCulture), viewComponentFactory);
			}	
		}
	}
}
