﻿using System;
using System.Reflection;
using System.Security;
using System.Web;
using System.Web.Security;
using Cuyahoga.Core.Service.Membership;
using Cuyahoga.Web.MVC;
using CuyahogaUser = Cuyahoga.Core.Domain.User;

namespace Cuyahoga.Web.Manager.Controllers
{
	public abstract class SecureController : BaseController
	{
		private ICuyahogaContext _cuyahogaContext;
		private IUserService _userService;

		public ICuyahogaContext CuyahogaContext
		{
			get { return this._cuyahogaContext; }
			set { this._cuyahogaContext = value; }
		}

		public IUserService UserService
		{
			protected get { return this._userService; }
			set { this._userService = value; }
		}

		/// <summary>
		/// Creates a new instance of the <see cref="SecureController"/> class.
		/// </summary>
		/// <param name="cuyahogaContext"></param>
		/// <param name="userService"></param>
		public SecureController()
		{
		}

		protected override bool OnPreAction(string actionName, System.Reflection.MethodInfo methodInfo)
		{
			// Note: this is more of a cross-cutting concern. In MonoRail we'd handle this with 
			// a filter or maybe even a Windsor Facility.

			// First check if we have to deal with an authenticated user
			HttpCookie authCookie = Request.Cookies[FormsAuthentication.FormsCookieName];
			if (authCookie == null)
			{
				SendToLoginPage();
				return false;
			}
			string cookieContent = authCookie.Value;

			try
			{
				// First, call the base OnPreAction because it sets the Message context.
				if (!base.OnPreAction(actionName, methodInfo))
				{
					return false;
				}

				// Read the cookie, get the Cuyahoga user and attach it to the context.
				FormsAuthenticationTicket ticket = FormsAuthentication.Decrypt(cookieContent);

				int userId = Convert.ToInt32(ticket.Name);

				CuyahogaUser cuyahogaUser = this._userService.GetUserById(userId);
				cuyahogaUser.IsAuthenticated = true;
				this._cuyahogaContext.SetUser(cuyahogaUser);
				ViewData["CuyahogaUser"] = cuyahogaUser;

				// Now check if there are any permission attributes specified.
				try
				{
					CheckPermissions(actionName, methodInfo);
				}
				catch (SecurityException ex)
				{
					ShowException(ex);
					RenderView("Error");
					return false;
				}

				return true;
			}
			catch (Exception ex)
			{
				Logger.Error("Error performing authorization check", ex);
				throw;
			}
		}

		private void SendToLoginPage()
		{
			string returnUrl = Request.RawUrl;
			Response.Redirect("~/Login/Index?returnurl=" + returnUrl);
		}

		private void CheckPermissions(string actionName, MethodInfo method)
		{
			// TODO: cache permission attributes.
			object[] attributes = method.GetCustomAttributes(typeof(CuyahogaPermissionAttribute), true);
			if (attributes.Length > 0)
			{
				CuyahogaPermissionAttribute permissionAttribute = (CuyahogaPermissionAttribute)attributes[0];

				// Permission attribute found, check if the user has one of the required rights.
				CheckPermission(actionName, permissionAttribute);
			}
			else
			{
				// Permission not found, check if there is an attribute on controller level
				object[] classAttributes = this.GetType().GetCustomAttributes(typeof(CuyahogaPermissionAttribute), true);
				if (classAttributes.Length > 0)
				{
					CuyahogaPermissionAttribute permissionAttribute = (CuyahogaPermissionAttribute)classAttributes[0];
					CheckPermission(actionName, permissionAttribute);
				}
			}
		}

		private void CheckPermission(string actionName, CuyahogaPermissionAttribute permissionAttribute)
		{
			bool isAllowed = true;

			foreach (string right in permissionAttribute.RightsArray)
			{
				if (!this._cuyahogaContext.CurrentUser.HasRight(right))
				{
					isAllowed = false;
				}
			}
			if (!isAllowed)
			{
				if (Logger.IsErrorEnabled)
				{
					Logger.ErrorFormat("User '{0}' is not allowed to perform the action '{1}' that requires right(s) '{2}'.", this._cuyahogaContext.CurrentUser.UserName, actionName, permissionAttribute.RequiredRights);
				}
				throw new SecurityException("ActionNotAllowedException");
			}
		}
	}
}
