﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using System.Web.Mvc.BindingHelpers;
using Castle.Components.Validator;
using Cuyahoga.Core.Domain;
using Cuyahoga.Core.Util;
using CuyahogaUser = Cuyahoga.Core.Domain.User;
using Cuyahoga.Core.Service.Membership;

namespace Cuyahoga.Web.Manager.Controllers
{
	public class UsersController : SecureController
	{

		[ControllerAction]
		public void Index()
		{
			RenderView("Index");
		}

		[ControllerAction]
		public void Find(string searchName)
		{
			ViewData["searchname"] = searchName;
			IList<User> users = base.UserService.FindUsersByName(searchName);
			if (users.Count > 0)
			{
				ViewData["users"] = users;
			}
			else
			{
				ShowMessage(Resources.GlobalResources.NoUsersFound);
			}
			RenderView("Index");
		}

		[ControllerAction]
		public void NewUser()
		{
			if (TempData.ContainsKey("user"))
			{
				ViewData["user"] = TempData["user"];
			}
			else
			{
				ViewData["user"] = new User();
			}
			ViewData["roles"] = base.UserService.GetAllRoles();
			RenderView("NewUser");
		}

		[ControllerAction]
		public void CreateUser(string userName, string email, string roleIds)
		{
			CuyahogaUser user = new CuyahogaUser();
			user.UserName = userName;
			user.Email = email;
			string newPassword = user.GeneratePassword();
			if (!String.IsNullOrEmpty(roleIds))
			{
				user.Roles = base.UserService.GetRolesByIds(Text.ExtractIntegersFromCommaSeparatedString(roleIds));
			}
			if (ValidatorRunner.IsValid(user))
			{
				base.UserService.UpdateUser(user);
				ShowMessage(String.Format(Resources.GlobalResources.NewUserMessage, userName, newPassword), true);
				Response.Redirect("~/Users/Find?searchname=" + userName);
			}
			else
			{
				ShowError(ValidatorRunner.GetErrorSummary(user), true);
				TempData["user"] = user;
				RedirectToAction("NewUser");
			}
		}

		[ControllerAction]
		public void EditUser(int id)
		{
			if (TempData.ContainsKey("user"))
			{
				ViewData["user"] = TempData["user"];
			}
			else
			{
				ViewData["user"] = base.UserService.GetUserById(id);
			}
			ViewData["roles"] = base.UserService.GetAllRoles();
			RenderView("EditUser");
		}

		[ControllerAction]
		public void UpdateUser(int userId)
		{
			User user = base.UserService.GetUserById(userId);
			user.UpdateFrom(Request.Form);
			user.Roles.Clear();
			string roleIds = Request.Form["RoleIds"];
			if (!String.IsNullOrEmpty(roleIds))
			{
				user.Roles = base.UserService.GetRolesByIds(Text.ExtractIntegersFromCommaSeparatedString(roleIds));
			}
			if (ValidatorRunner.IsValid(user))
			{
				base.UserService.UpdateUser(user);
				ShowMessage(Resources.GlobalResources.UserUpdated, true);
				Response.Redirect("~/Users/Find?searchname=" + user.UserName);
			}
			else
			{
				ShowError(ValidatorRunner.GetErrorSummary(user), true);
				TempData["user"] = user;
				RedirectToAction(new { action = "EditUser", id = user.Id });
			}
		}

		[ControllerAction]
		public void UpdatePassword(int userId, string password, string passwordConfirmation)
		{
			User user = base.UserService.GetUserById(userId);
			user.Password = String.IsNullOrEmpty(password) ? String.Empty : Encryption.StringToMD5Hash(password);
			user.PasswordConfirmation = String.IsNullOrEmpty(passwordConfirmation) ? String.Empty : Encryption.StringToMD5Hash(passwordConfirmation);
			if (ValidatorRunner.IsValid(user, RunWhen.Custom))
			{
				base.UserService.UpdateUser(user);
				ShowMessage(Resources.GlobalResources.PasswordUpdated, true);
				Response.Redirect("~/Users/Find?searchname=" + user.UserName);
			}
			else
			{
				ShowError(ValidatorRunner.GetErrorSummary(user), true);
				TempData["user"] = user;
				RedirectToAction(new { action = "EditUser", id = user.Id });
			}
		}

		[ControllerAction]
		[CuyahogaPermission(RequiredRights="Delete User")]
		public void DeleteUser(int userId)
		{
			if (userId == CuyahogaContext.CurrentUser.Id)
			{
				ShowError(Resources.GlobalResources.DeleteOwnAccountForbidden, true);
			}
			else
			{
				User user = base.UserService.GetUserById(userId);
				base.UserService.DeleteUser(user);
				ShowMessage(String.Format(Resources.GlobalResources.UserDeleted, user.UserName), true);
			}
			RedirectToAction("Index");
		}
	}
}
