﻿using System;
using System.Web;
using System.Web.Mvc;
using Castle.Core;
using Castle.Windsor;
using Castle.Windsor.Configuration.Interpreters;
using Cuyahoga.Core.Util;
using Cuyahoga.Web.Manager.Controllers;
using Cuyahoga.Web.Manager.ViewComponents;
using Cuyahoga.Web.MVC;
using Cuyahoga.Web.MVC.ViewComponents;
using Cuyahoga.Web.MVC.Windsor;
using Castle.Components.Validator;

namespace Cuyahoga.Web.Manager
{
	public class Global : System.Web.HttpApplication, IContainerAccessor
	{
		public IWindsorContainer Container
		{
			get { return IoC.Container; }
		}

		protected void Application_Start(object sender, EventArgs e)
		{
			// Create the Windsor IoC container.
			IWindsorContainer container = new WindsorContainer(new XmlInterpreter());
			// Initialize the static IoC class that is being used to access components that are registered in Windsor.
			IoC.Initialize(container);
			// Add ICuyahogaContext to the container.
			container.AddComponentWithLifestyle("cuyahoga.context", typeof(ICuyahogaContext), typeof(CuyahogaContext), LifestyleType.PerWebRequest);
			// Add WindsorViewComponentFactory to the container.
			container.AddComponent("cuyahoga.viewcomponentfactory", typeof(WindsorViewComponentFactory));
			// Add Validator registry
			container.AddComponent("castle.validatorregistry", typeof(IValidatorRegistry), typeof(CachedValidationRegistry));
			// Add controllers to the container.
			AddControllers();
			// Add view components to the container
			AddViewComponents();
			// Add routes.
			AddRoutes();
		}

		protected void Application_BeginRequest(Object sender, EventArgs e)
		{
			// Initialize CuyahogaContext.
			ICuyahogaContext cuyahogaContext = Container.Resolve<ICuyahogaContext>();
			cuyahogaContext.Initialize(HttpContext.Current);
		}

		/// <summary>
		/// Add Controllers to the container. Also associate the Controller 
		/// with the WindsorContainer ControllerFactory.
		/// </summary>
		protected virtual void AddControllers()
		{
			if (Container == null)
			{
				throw new InvalidOperationException("Unable to add controllers to the container because the container is unavailable.");
			}
			Type[] assemblyTypes = typeof(BaseController).Assembly.GetTypes();
			foreach (Type type in assemblyTypes)
			{
				if (typeof(IWindsorController).IsAssignableFrom(type))
				{
					Container.AddComponentWithLifestyle(type.Name, type, LifestyleType.Transient);
					ControllerBuilder.Current.SetControllerFactory(type, typeof(WindsorControllerFactory));
				}
			}
		}

		/// <summary>
		/// Add view components to the container that should be managed by Windsor.
		/// </summary>
		protected virtual void AddViewComponents()
		{
			if (Container == null)
			{
				throw new InvalidOperationException("Unable to add view components to the container because the container is unavailable.");
			}
			Container.AddComponentWithLifestyle("viewcomponent.mainmenu", typeof(MainMenuComponent), LifestyleType.Transient);
			// Register view component with factory. We could also set the default view component factory to WindsorViewComponentFactory.
			// Another way to register view components is via the configuration. In this case, the default view component factory
			// MUST be set to the WindsorViewComponentFactory.
			ViewComponentBuilder.Current.SetViewComponentFactory("MainMenu", Container.Resolve<WindsorViewComponentFactory>());
		}

		protected virtual void AddRoutes()
		{
			RouteTable.Routes.Add(new Route
			{
				Url = "[controller]/[action]/[id]",
				Defaults = new { action = "Index", id = (string)null },
				RouteHandler = typeof(MvcRouteHandler)
			});

			RouteTable.Routes.Add(new Route
			{
				Url = "Default.aspx",
				Defaults = new { controller = "Dashboard", action = "Index", id = (string)null },
				RouteHandler = typeof(MvcRouteHandler)
			});
		}

	}
}